#!/usr/bin/env python
#--------------------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2008
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program License Agreement.
# See the software licensing materials that came with the IBM XIV Storage System for terms and conditions.
#
#--------------------------------------------------------------------------------------------------------------


from host_attach import logger
from platform import system
from sys import exit
from host_attach import logger, is_error_free, execute_and_log, PACKAGE_DIR

DRIVERS_PATH = PACKAGE_DIR + '/drivers/linux/emulex'
EMULEX_DRIVER_PATH = DRIVERS_PATH + '/lpfc_2.6_driver_kit-8.2.0.29-1.tar.gz'
EMULEX_LUN_SCAN_SCRIPT = PACKAGE_DIR + '/bin/emulex_lun_scan'
EMULEX_MINIMAL_DRIVER_VERSION  = '0:8.2.0'

def emulex_get_scsi_hosts():
	hosts = []
	output = execute_and_log('find /sys/class/scsi_host/ -type f | grep lpfc_drvr_version')
	for node in output.split('/'):
		if node.__contains__('host') and not node.__contains__('scsi_host'):
			hosts.append(node)
	return hosts

def emulex_get_driver_string(host):
	return execute_and_log('cat /sys/class/scsi/scsi_host/' + host + '/lpfc_drvr_version')

def emulex_get_driver_version_from_driver_string(string=''):
	elements = string.split()
	count = 0
	for element in elements:
		count = count +1
	return elements[count-1]

def emulex_get_driver_version():
	hosts = emulex_get_scsi_hosts()
	host = hosts[0]
	driver_string = emulex_get_driver_string(host)
	driver_version = emulex_get_driver_version_from_driver_string(driver_string)
	return driver_version

def emulex_install():
        file_path_without_extension = DRIVERS_PATH + '/lpfc_2.6_driver_kit-8.2.0.29-1'
        output = execute_and_log('tar -zxf ' + EMULEX_DRIVER_PATH + ' -C ' + DRIVERS_PATH)
        if not is_error_free(output):
                logger.debug('failed to extract ' + EMULEX_DRIVER_PATH)
                return 0
        output = execute_and_log('cd ' + file_path_without_extension + ' && ./lpfc-install')
        if not is_error_free(output):
                logger.error('emulex installation script returned an error, check for error in logfile /var/log/host_attach')
                execute_and_log('popd')
                return 0
	execute_and_log('popd')
        return 1

def emulex_upgrade():
	return emulex_install()

def is_emulex_driver_installed():
	output = execute_and_log('ls /lib/modules/`uname -r`/kernel/drivers/scsi/lpfc')
	if not is_error_free(output):
		logger.debug('emulex driver is not installed')
		return 0
	return 1
	
def is_emulex_driver_loaded():
	output = execute_and_log('lsmod | grep lpfc')
	if not is_error_free(output):
		logger.debug('emulex driver lpfc not loaded')
		return 0
	return 1

def is_emulex_driver_up2date():
	driver_version = emulex_get_driver_version()
	if driver_version < EMULEX_MINIMAL_DRIVER_VERSION:
		logger.debug('the installed emulex driver ' + driver_version + 'is older then required: ' + EMULEX_MINIMAL_DRIVER_VERSION)
		return 0
	return 1

def emulex_verify():
	if not is_emulex_driver_installed():
		logger.error('emulex driver is not installed, use --install')
		return 0
	if not is_emulex_driver_loaded():
		logger.error('emulex driver is not loaded, probably not installed')
		return 0
	if not is_emulex_driver_up2date():
		logger.error('installed emulex driver version does not meet minimal required version, use --upgrade')
		return 0
	logger.debug('the installed emulex driver is recent enough')
	return 1

def emulex_uninstall():
	file_path_without_extension = DRIVERS_PATH + '/lpfc_2.6_driver_kit-8.2.0.29-1'
	output = execute_and_log('tar -zxf ' + EMULEX_DRIVER_PATH + ' -C ' + DRIVERS_PATH)
	if not is_error_free(output):
		logger.debug('failed to extract ' + EMULEX_DRIVER_PATH)
		return 0
	output = execute_and_log('cd ' + file_path_without_extension + ' && ./lpfc-install --uninstall')
	if not is_error_free(output):
		logger.error('emulex installation script returned an error, check for error in logfile /var/log/host_attach')
		execute_and_log('popd')
		return 0
        execute_and_log('popd')
	return 1

def emulex_scan():
	output = execute_and_log(EMULEX_LUN_SCAN_SCRIPT + ' all')
	if not is_error_free(output):
		logger.debug ('failed to execute ' + EMULEX_LUN_SCAN_SCRIPT + ' all')
		return 0
	return 1
