#!/usr/bin/env python
#--------------------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2008
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program License Agreement.
# See the software licensing materials that came with the IBM XIV Storage System for terms and conditions.
#
#--------------------------------------------------------------------------------------------------------------

import host_attach
from host_attach import logger, is_error_free, execute_and_log, PACKAGE_DIR
from platform import system, dist
from sys import exit

def get_device_from_fstab_line(line):
	return line.split()[0]

def get_mpoint_from_fstab_line(line):
	return line.split()[1]

def get_mounts():
	fstab = execute_and_log('cat /etc/fstab | grep -v "#"')
	fstab_lines = fstab.splitlines()
	mounts = {}
	for line in fstab_lines:
		device = get_device_from_fstab_line(line)
		mpoint = get_mpoint_from_fstab_line(line)
		mounts[mpoint]=device
	return mounts

def is_boot_critical(mpoint):
	boot_mount_points = ['/','/boot','/var','/etc']
	if boot_mount_points.__contains__(mpoint):
		return 1
	return 0

def is_label(mpoint):
	if mpoint.__contains__('LABEL'):
		return 1
	return 0

def get_device_from_label(label):
	output = execute_and_log('findfs ' + label)
	if not is_error_free(output):
		logger.debug('failed to find device from label ' + label)
		return 0
	return output

def is_logical_volume(mpoint):
	if mpoint.__contains__('LogVol'):
		return 1
	return 0

def get_vg_name_from_lv_name(lv_name):
	lv_string = lv_name.split('/').pop()
	vg_name = lv_string.split('-')[0]
	return vg_name

def get_device_from_lvm(lv_name):
	vg_name = get_vg_name_from_lv_name(lv_name)
	pv_string = getoutput('lvm pvdisplay | grep ' + vg_name + ' -B 1 | head -1')
	device = pv_string.split().pop()
	return device

def is_2810_device(device):
	output = execute_and_log('sg_inq -i ' + device)
	if output.__contains__('2810'):
		return 1
	return 0

def is_device_multipathed(device):
	if device.__contains__('mapper'):
		return 1
	return 0

def is_boot_from_san(strict='no'):
	mounts = get_mounts()
	for mpoint in mounts:
		if is_boot_critical(mpoint):
			device = mounts[mpoint]
			if is_label(device):
				device = get_device_from_label(device)
			elif is_logical_volume(device):
				device  = get_device_from_lvm(device)
			if not device:
				logger.debug('failed to query label/lvm')
				return 0
			if is_2810_device(device):
				if strict == 'multipath' and not is_device_multipathed(device):
					return 0
				return 1
	return 0

def is_boot_from_san_supported():
	if (dist().__contains__('redhat') or dist().__contains__('centos')) and (dist().__contains__('5,1') or dist().__contains__('5.2')):
		return 1
	return 0

#def backup_fstab():
#	exectue_and_log('cp -rf /etc/fstab{,`date`}')
#	return 1
#
#def set_label(label, device):
#	label = label.replace('LABEL=','')
#	execute_and_log('e2label ' + label + ' ' + device)
#	return 1
#
#def regex_ready(before):
#	after = ''
#	for character in before:
#		if character == '/':
#			after.append('\\')
#		after.append(character)
#	return after
#
#def sed_fstab(old,new):
#	regex_old = regex_ready(old)
#	regex_new = regex_read(new)
#	execute_and_log('''cat /etc/fstab | sed 's/''' + regex_old + '/' + regex_new + '''/' > /etc/fstab''')
#	return 1
#
#def replace_in_fstab(original_device, mpath):
#	if is_label(original_device):
#		set_label(original_device,mpath)
#	else:
#		sed_fstab(original_device,mpath)
#	return 1
#
#def multipath_boot_from_san():
#	backup_fstab()
#	mounts = get_mounts()
#	for mpoint in mounts:
#		if is_boot_critical(mpoint):
#			original_device = mounts[mpoint]
#			device = mounts[mpoint]
#			if is_label(device):
#				device = get_device_from_label(device)
#			elif is_logical_volume(device):
#				device  = get_device_from_lvm(device)
#			if not device:
#				logger.debug('failed to query label/lvm')
#				return 0
#			if is_2810_device(device):
#				mpath = get_device_mpath(device)
#				if mpath == 0 or not is_error_free(mpath):
#					logger.error('failed to query mpath for device ' + device + 'which is in /etc/fstab ' + original_device)
#					return 0
#				if not replace_in_fstab(original_device, mpath):
#					logger.error('failed to update /etc/fstab, examine it manually before rebooting')
#					return 0
#	return 1


